#include "wx/wx.h"
#include <wx/glcanvas.h>
#include <GL/glu.h>

GLfloat rx,ry,rz;
GLfloat tx,ty,tz;
GLfloat sx,sy,sz;
bool rotacao,translacao,escala;
bool x,y,z;


class MyApp : public wxApp
{
public:
    virtual bool OnInit();
};

class TestGLCanvas;


class MyFrame : public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
            const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);
    TestGLCanvas* m_canvas;

    DECLARE_EVENT_TABLE()
};


class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:
 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas");
   
 ~TestGLCanvas(void);

 void OnChar(wxKeyEvent& event);
 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 
 void Render( void );
 void InitGL(void);
   
private:
  bool   m_init;


DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(TestGLCanvas, wxGLCanvas)
  EVT_SIZE(TestGLCanvas::OnSize)
  EVT_PAINT(TestGLCanvas::OnPaint)
  EVT_CHAR(TestGLCanvas::OnChar)
  EVT_ERASE_BACKGROUND(TestGLCanvas::OnEraseBackground)
  EVT_ENTER_WINDOW( TestGLCanvas::OnEnterWindow )
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
END_EVENT_TABLE()



IMPLEMENT_APP(MyApp)


static void Reshape(int width, int height)
{
  glViewport(0,0,width,height); 
  glMatrixMode( GL_PROJECTION ); 
  glLoadIdentity(); 

  gluPerspective(30.0,width/height,1.0,10.0); 



}



bool MyApp::OnInit()
{

  MyFrame *frame = new MyFrame(NULL, "Exemplo Basico", wxPoint(50, 50),
                               wxSize(350, 350));

  wxMenu *winMenu = new wxMenu;

  winMenu->Append(wxID_EXIT, "&Fechar");

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(winMenu, "&Janela");

  frame->SetMenuBar(menuBar);

  frame->m_canvas = new TestGLCanvas(frame, -1, wxPoint(0, 0), wxSize(500, 500));

  frame->Show(TRUE);

  tx = ty = tz = 0;
  sx = sy = sz = 1;  
  rx = 30;
  ry = -45;
  rz = 30;
  x = TRUE;
  y = FALSE;
  z = FALSE;
  rotacao = TRUE;
  translacao = FALSE;
  escala = FALSE;

  return TRUE;
}


MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                 const wxSize& size, long style) 
         : wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = NULL;
}

TestGLCanvas::TestGLCanvas(wxWindow *parent, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name):
  wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}


TestGLCanvas::~TestGLCanvas()
{
}

void MyFrame::OnExit(wxCommandEvent& event)
{
    Destroy();
}

void TestGLCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

void TestGLCanvas::OnEraseBackground(wxEraseEvent& event)
{
}

void TestGLCanvas::OnPaint( wxPaintEvent& event )
{
    Render();
    SwapBuffers();

}

void TestGLCanvas::Render()
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();

    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }
  
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
          glClear(GL_COLOR_BUFFER_BIT );
          glClearColor(0.0f,0.0f,0.0f,0.0f); 
          glColor3f(0.2, 0.2, 0.2);

          glMatrixMode( GL_MODELVIEW ); 
          glLoadIdentity();
          glTranslatef(0.0f,0.0f,-3.0f); 
          glRotatef(rx,1.0f,0.0f,0.0f); 
          glRotatef(ry,0.0f,1.0f,0.0f); 
          glRotatef(rz,0.0f,0.0f,1.0f); 
          glTranslatef(tx,ty,tz);
          glScalef(sx,sy,sz);
 
          glBegin(GL_TRIANGLE_FAN);
                glVertex3d(0.4, 0.8, 0.4);
                glColor3f(0.3, 0.4, 0.5);
                glVertex3d(0.4, 0.0, 0.0);
                glVertex3d(0.0, 0.0, 0.8);
                glVertex3d(0.8, 0.0, 0.8);
                glVertex3d(0.4, 0.0, 0.0);
         glEnd();

       
        glColor3f(1.0, 0.7, 0.5);

        // Draw the base of the pyramid
        glBegin(GL_TRIANGLES);
                glVertex3d(0.4, 0.0, 0.0);
                glVertex3d(0.0, 0.0, 0.8);
                glVertex3d(0.8, 0.0, 0.8);
        glEnd();
      
        glTranslatef(0.0f,0.0f,-1.5);
          glBegin(GL_TRIANGLE_FAN);
                glVertex3d(0.4, 0.8, 0.4);
                glColor3f(0.3, 0.4, 0.5);
                glVertex3d(0.4, 0.0, 0.0);
                glVertex3d(0.0, 0.0, 0.8);
                glVertex3d(0.8, 0.0, 0.8);
                glVertex3d(0.4, 0.0, 0.0);
         glEnd();


       
        glColor3f(0.2, 0.3, 0.5);

        // Draw the base of the pyramid
        glBegin(GL_TRIANGLES);
                glVertex3d(0.4, 0.0, 0.0);
                glVertex3d(0.0, 0.0, 0.8);
                glVertex3d(0.8, 0.0, 0.8);
        glEnd();


  glFlush();
  SwapBuffers();
}

void TestGLCanvas::InitGL()
{
   SetCurrent();
   glClearColor(0.0, 0.0, 0.0, 0.0);
   glEnable(GL_DEPTH_TEST);


}

void TestGLCanvas::OnSize(wxSizeEvent& event)
{
    int width, height;
    GetClientSize(& width, & height);
    Reshape(width, height);
        if (height==0)  
	{ 
                height=1;   
	}  


#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
    }
}

void TestGLCanvas::OnChar(wxKeyEvent& event)
{
    switch(event.KeyCode()) {
    case '1':
           rotacao = TRUE;
           translacao = FALSE;
           escala = FALSE;
           break;
    case '2':
           translacao = TRUE;
           rotacao = FALSE;
           escala = FALSE;
           break;
    case '3':
           rotacao = FALSE;
           translacao = FALSE;
           escala = TRUE;
           break;
    case 'x': case 'X':
           x = TRUE;
           y = FALSE;
           z=  FALSE;
           break;
    case 'y': case 'Y':
           y = TRUE;
           x = FALSE;
           z=  FALSE;
           break;
    case 'z': case 'Z':
           z = TRUE;
           y = FALSE;
           x=  FALSE;
           break;
    case WXK_ESCAPE:
	exit(0);
    case WXK_LEFT:
        if (x && rotacao)
           rx += 3.0;
        else if (y && rotacao)
           ry += 3.0;
        else if (z && rotacao) 
           rz += 3.0;
        else if (x && translacao)
           tx += 0.1;
        else if (y && translacao)
           ty += 0.1;
        else if (z && translacao) 
           tz += 0.1;
        else if (x && escala)
           sx += 0.1;
        else if (y && escala)
           sy += 0.1;
        else if (z && escala) 
           sz += 0.1;
	break;
    case WXK_RIGHT:
        if (x && rotacao)
           rx -= 3.0;
        else if (y && rotacao)
           ry -= 3.0;
        else if (z && rotacao) 
           rz -= 3.0;
        else if (x && translacao)
           tx -= 0.1;
        else if (y && translacao)
           ty -= 0.1;
        else if (z && translacao) 
           tz -= 0.1;
        else if (x && escala)
           sx -= 0.1;
        else if (y && escala)
           sy -= 0.1;
        else if (z && escala) 
           sz -= 0.1;
	break;
/*    case WXK_UP:
	angulo = angulo + 3.0;
	break;
    case WXK_DOWN:
	angulo = angulo - 3.0;
	break;
    case 's': case 'S':
	smooth = !smooth;
	if (smooth) {
	    glShadeModel(GL_SMOOTH);
	} else {
	    glShadeModel(GL_FLAT);
	}
	break;
    case 'l': case 'L':
	lighting = !lighting;
	if (lighting) {
	    glEnable(GL_LIGHTING);
	} else {
	    glDisable(GL_LIGHTING);
	}
	break;*/

     default:
      {
        event.Skip();
        return;
      }
    }

    Refresh(FALSE);
}

